/*
 This file is part of digitalStrom NetAtmo Virtual Device and needs
 libdSvDC from https://gitorious.digitalstrom.org/virtual-devices/libdsvdc.

 digitalStrom NetAtmo Virtual Device is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 libdsvdc is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/

#ifdef HAVE_CONFIG_H
    #include "config.h"
#endif

#include "netatmo.h"

#include <stdarg.h>
#include <stdio.h>
#include <sys/time.h>
#include <string.h>
#include <pthread.h>
#include <inttypes.h>
#include <ctype.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>


static pthread_mutex_t reportMutex;
static int debugLevel = LOG_WARNING;
static void printMessage(char *buf);

static double get_timestamp(void)
{
  struct timeval tv;

  gettimeofday(&tv, NULL);
  return (tv.tv_sec + tv.tv_usec*1e-6);
}

void vdc_init_report()
{
  pthread_mutex_init(&reportMutex, NULL);
}

void vdc_set_debugLevel(int debug)
{
  debugLevel = debug;
}

int vdc_get_debugLevel()
{
  return debugLevel;
}

void vdc_report(int errlevel, const char *fmt, ... )
{
  if (errlevel <= debugLevel) {
    char buf[BUFSIZ];
    va_list ap;
    pthread_mutex_lock(&reportMutex);

    strncpy(buf, "netatmo: ", BUFSIZ);
    va_start(ap, fmt);
    vsnprintf(buf + strlen(buf), sizeof(buf)-strlen(buf), fmt, ap);
    va_end(ap);

    printMessage(buf);

    pthread_mutex_unlock(&reportMutex);
  }
}

void vdc_report_extraLevel(int errlevel, int maxErrlevel, const char *fmt, ... )
{
  if (errlevel <= maxErrlevel) {
    char buf[BUFSIZ];
    va_list ap;
    pthread_mutex_lock(&reportMutex);

    strncpy(buf, "netatmo: ", BUFSIZ);
    va_start(ap, fmt);
    vsnprintf(buf + strlen(buf), sizeof(buf)-strlen(buf), fmt, ap);
    va_end(ap);

    printMessage(buf);

    pthread_mutex_unlock(&reportMutex);
  }
}

static void printMessage(char *buf)
{
  char buf2[BUFSIZ], *sp, tsbuf[30];
  struct timeval t;
  gettimeofday(&t, NULL);
  strftime(tsbuf, sizeof(tsbuf), "%Y-%m-%d %H:%M:%S", localtime(&t.tv_sec));
  (void)snprintf(buf2, BUFSIZ, "[%s.%03d] ", tsbuf, (int)t.tv_usec / 1000);
  for (sp = buf; *sp != '\0'; sp++)
    if (isprint(*sp) || (isspace(*sp) && (sp[1]=='\0' || sp[2]=='\0')))
      (void)snprintf(buf2+strlen(buf2), 2, "%c", *sp);
    else
      (void)snprintf(buf2+strlen(buf2), 6, "\\x%02x", (unsigned)*sp);
  (void)fputs(buf2, stderr);
}
